import time
import os
from windproapi.utils import get_windpro_sample_path
from windproapi import nan_to_skipvalue
from windproapi import WindProApi
from windproapi import compare_objects
from copy import deepcopy

# Opening windPRO
_windproapi = WindProApi()
working_dir = os.path.join(get_windpro_sample_path('3.6'), 'Ebeltoft - Denmark\\3.6')
project_path = os.path.join(working_dir, 'DEMO - Ebeltoft, DK.w36p')

_windproapi.start_windpro_random_port()

# Services
project_service = _windproapi.get_service('ProjectService')
objects_service = _windproapi.get_service('ObjectsService')
obj_NSA_service = _windproapi.get_service('ObjNSAService')
factory = _windproapi.get_factory('WindproService')

# Loading project
project_service.LoadFromFile(filename=project_path)

# Getting all Noise Sensitive Area objects
objs = objects_service.GetObjects(apiObjType='NSA')

# Loading are object specific data
nsa_obj = obj_NSA_service.GetNSAObject(objs[0].Handle)

# Changing the shape of the noise sensitive area
nsa_obj.Points.TApiGP[0].Lat = nsa_obj.Points.TApiGP[0].Lat + 0.03

# Changing the noise demand. Caution, this will not work as this object as a predefined noise level level set!
nan_to_skipvalue(nsa_obj)
obj_NSA_service.SetNSAObject(nsa_obj)

# Comparing object that was set with what is in windPRO
nsa_obj_wp = obj_NSA_service.GetNSAObject(objs[0].Handle)
compare_objects(nsa_obj_wp, nsa_obj)

# Adding a new noise recepors
# Set those with a constant freely defined noise constraint

# Single noise receptor
new_obj = objects_service.AddObject(apiObjType='NSA',
                                    lat=56.17,
                                    lng=10.66,
                                    userDesc='Script Added Point')
nsa_obj = obj_NSA_service.GetNSAObject(new_obj.Handle)
nsa_obj_default = deepcopy(nsa_obj)

# Changing noise level to a user defined absolute noise
nsa_obj.FreeDefinable = True
nsa_obj.KindOfDemand = 'kodAbsolute'
nsa_obj.NoiseDemandNew = 37
nan_to_skipvalue(nsa_obj)
obj_NSA_service.SetNSAObject(nsa_obj)

print(compare_objects(nsa_obj, nsa_obj_default))

# Adding NSA with area
new_obj = objects_service.AddObject(apiObjType='NSA',
                                    lat=56.17,
                                    lng=10.67,
                                    userDesc='Script Added Area')
nsa_obj = obj_NSA_service.GetNSAObject(new_obj.Handle)

# Adding points to the area
for i_lat, i_lng in zip([56.172, 56.172, 56.162, 56.162], [10.67, 10.68, 10.68, 10.675]):
    point = factory.TApiGP()
    point.Lat = i_lat
    point.Lng = i_lng
    nsa_obj.Points.TApiGP.append(point)

nsa_obj.FreeDefinable = True
nsa_obj.KindOfDemand = 'kodAbsolute'
nsa_obj.NoiseDemandNew = 42
nan_to_skipvalue(nsa_obj)
obj_NSA_service.SetNSAObject(nsa_obj)
