import time
import os
from windproapi.utils import get_windpro_sample_path
from windproapi import WindProApi

# Opening windPRO
_windproapi = WindProApi()

working_dir = os.path.join(get_windpro_sample_path('3.6'), 'New Salem\\3.6')
project_path = os.path.join(working_dir, 'New Salem.w36p')

_windproapi.start_windpro_random_port()

# Services
project_service = _windproapi.get_service('ProjectService')
factory = _windproapi.get_factory('WindproService')

# Loading project
project_service.LoadFromFile(filename=project_path)

# Getting the current WAsP setup
wasp_params = project_service.GetProjectWaspParams()
print(wasp_params)

# Printing all descriptions and their associated IDs
for i, d in enumerate(wasp_params.WaspParams.TApiWaspConfItem):
    print(i, d.Description, d.ID)

# modifying the heat flux over over land
for conf in wasp_params.WaspParams.TApiWaspConfItem:
    if conf['ID'] == 'OFSLAND':
        print(conf)
        conf.Value = -70

for conf in wasp_params.WaspParams.TApiWaspConfItem:
    if conf['ID'] == 'OFSLAND':
        print(conf)

# Setting the values in windPRO
project_service.SetProjectWaspParams(params=wasp_params)

# Confirming that the WAsP paramter has been set correctly.
wasp_params_new = project_service.GetProjectWaspParams()
for conf in wasp_params_new.WaspParams.TApiWaspConfItem:
    if conf['ID'] == 'OFSLAND':
        print(conf)
